@@ -34,17 +34,18 @@ module Hpack.Render (
 #endif
 ) where
 
-import           Control.Monad
+import           Imports
+
 import           Data.Char
 import           Data.Maybe
-import           Data.List
 import           Data.Map.Lazy (Map)
 import qualified Data.Map.Lazy as Map
 
 import           Hpack.Util
 import           Hpack.Config
 import           Hpack.Render.Hints
-import           Hpack.Render.Dsl
+import           Hpack.Render.Dsl hiding (sortFieldsBy)
+import qualified Hpack.Render.Dsl as Dsl
 
 renderPackage :: [String] -> Package -> String
 renderPackage oldCabalFile = renderPackageWith settings alignment formattingHintsFieldOrder formattingHintsSectionsFieldOrder
@@ -65,7 +66,8 @@ renderPackageWith settings headerFieldsAlignment existingFieldOrder sectionsFiel
     packageFields :: [Element]
     packageFields = addVerbatim packageVerbatim . sortFieldsBy existingFieldOrder $
       headerFields ++ [
-        Field "extra-source-files" (renderPaths packageExtraSourceFiles)
+        Field "tested-with" $ CommaSeparatedList packageTestedWith
+      , Field "extra-source-files" (renderPaths packageExtraSourceFiles)
       , Field "extra-doc-files" (renderPaths packageExtraDocFiles)
       , Field "data-files" (renderPaths packageDataFiles)
       ] ++ maybe [] (return . Field "data-dir" . Literal) packageDataDir
@@ -108,7 +110,6 @@ renderPackageWith settings headerFieldsAlignment existingFieldOrder sectionsFiel
       , case packageLicenseFile of
           [file] -> ("license-file", Just file)
           files  -> ("license-files", formatList files)
-      , ("tested-with", packageTestedWith)
       , ("build-type", Just (show packageBuildType))
       ]
 
@@ -165,7 +166,7 @@ renderExecutables :: Map String (Section Executable) -> [Element]
 renderExecutables = map renderExecutable . Map.toList
 
 renderExecutable :: (String, Section Executable) -> Element
-renderExecutable (name, sect@(sectionData -> Executable{..})) =
+renderExecutable (name, sect) =
   Stanza ("executable " ++ name) (renderExecutableSection [] sect)
 
 renderTests :: Map String (Section Executable) -> [Element]
@@ -286,7 +287,13 @@ renderConditional renderSectionData (Conditional condition sect mElse) = case mE
   Nothing -> if_
   Just else_ -> Group if_ (Stanza "else" $ renderSection renderSectionData [] [] else_)
   where
-    if_ = Stanza ("if " ++ condition) (renderSection renderSectionData [] [] sect)
+    if_ = Stanza ("if " ++ renderCond condition) (renderSection renderSectionData [] [] sect)
+
+renderCond :: Cond -> String
+renderCond = \ case
+  CondExpression c -> c
+  CondBool True -> "true"
+  CondBool False -> "false"
 
 defaultLanguage :: Element
 defaultLanguage = Field "default-language" "Haskell2010"
@@ -296,17 +303,17 @@ renderDirectories name = Field name . LineSeparatedList . replaceDots
   where
     replaceDots = map replaceDot
     replaceDot xs = case xs of
-      "." -> "./."
+      "." -> "./"
       _ -> xs
 
-renderExposedModules :: [String] -> Element
-renderExposedModules = Field "exposed-modules" . LineSeparatedList
+renderExposedModules :: [Module] -> Element
+renderExposedModules = Field "exposed-modules" . LineSeparatedList . map unModule
 
-renderOtherModules :: [String] -> Element
-renderOtherModules = Field "other-modules" . LineSeparatedList
+renderOtherModules :: [Module] -> Element
+renderOtherModules = Field "other-modules" . LineSeparatedList . map unModule
 
-renderGeneratedModules :: [String] -> Element
-renderGeneratedModules = Field "autogen-modules" . LineSeparatedList
+renderGeneratedModules :: [Module] -> Element
+renderGeneratedModules = Field "autogen-modules" . LineSeparatedList . map unModule
 
 renderReexportedModules :: [String] -> Element
 renderReexportedModules = Field "reexported-modules" . CommaSeparatedList
@@ -394,10 +401,10 @@ renderBuildable :: Bool -> Element
 renderBuildable = Field "buildable" . Literal . show
 
 renderDefaultExtensions :: [String] -> Element
-renderDefaultExtensions = Field "default-extensions" . WordList
+renderDefaultExtensions = Field "default-extensions" . LineSeparatedList
 
 renderOtherExtensions :: [String] -> Element
-renderOtherExtensions = Field "other-extensions" . WordList
+renderOtherExtensions = Field "other-extensions" . LineSeparatedList
 
 renderPaths :: [Path] -> Value
 renderPaths = LineSeparatedList . map renderPath
@@ -409,3 +416,6 @@ renderPaths = LineSeparatedList . map renderPath
 
     needsQuoting :: FilePath -> Bool
     needsQuoting = any (\x -> isSpace x || x == ',')
+
+sortFieldsBy :: [String] -> [Element] -> [Element]
+sortFieldsBy existingFieldOrder = Dsl.sortFieldsBy ("import" : existingFieldOrder)
