@@ -4,6 +4,7 @@ module Hpack.Render.Hints (
   FormattingHints (..)
 , sniffFormattingHints
 #ifdef TEST
+, sniffRenderSettings
 , extractFieldOrder
 , extractSectionsFieldOrder
 , sanitize
@@ -15,12 +16,13 @@ module Hpack.Render.Hints (
 #endif
 ) where
 
+import           Imports
+
 import           Data.Char
 import           Data.Maybe
-import           Data.List
-import           Control.Applicative
 
 import           Hpack.Render.Dsl
+import           Hpack.Util
 
 data FormattingHints = FormattingHints {
   formattingHintsFieldOrder :: [String]
@@ -109,6 +111,8 @@ sniffCommaStyle input
 sniffRenderSettings :: [String] -> RenderSettings
 sniffRenderSettings input = RenderSettings indentation fieldAlignment commaStyle
   where
-    indentation = fromMaybe (renderSettingsIndentation defaultRenderSettings) (sniffIndentation input)
+    indentation = max def $ fromMaybe def (sniffIndentation input)
+      where def = renderSettingsIndentation defaultRenderSettings
+
     fieldAlignment = renderSettingsFieldAlignment defaultRenderSettings
     commaStyle = fromMaybe (renderSettingsCommaStyle defaultRenderSettings) (sniffCommaStyle input)
