@@ -40,6 +40,28 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           Paths_foo
       |]
 
+    describe "tested-with" $ do
+      it "accepts a string" $ do
+        [i|
+        tested-with: GHC == 7.0.4
+        |] `shouldRenderTo` package [i|
+        tested-with:
+            GHC == 7.0.4
+        |]
+
+      it "accepts a list" $ do
+        [i|
+        tested-with:
+        - GHC == 7.0.4
+        - GHC == 7.2.2
+        - GHC == 7.4.2
+        |] `shouldRenderTo` package [i|
+        tested-with:
+            GHC == 7.0.4
+          , GHC == 7.2.2
+          , GHC == 7.4.2
+        |]
+
     it "warns on duplicate fields" $ do
       [i|
       name: foo
@@ -48,6 +70,95 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           "package.yaml: Duplicate field $.name"
         ]
 
+    describe "handling of Paths_ module" $ do
+      it "adds Paths_ to other-modules" $ do
+        [i|
+        library: {}
+        |] `shouldRenderTo` library [i|
+        other-modules:
+            Paths_foo
+        |]
+
+      context "when cabal-version is >= 2" $ do
+        it "adds Paths_ to autogen-modules" $ do
+          [i|
+          verbatim:
+            cabal-version: 2.0
+          library: {}
+          |] `shouldRenderTo` (library [i|
+          other-modules:
+              Paths_foo
+          autogen-modules:
+              Paths_foo
+          |]) { packageCabalVersion = "2.0" }
+
+        context "when Paths_ module is listed explicitly under generated-other-modules" $ do
+          it "adds Paths_ to autogen-modules only once" $ do
+            [i|
+            verbatim:
+              cabal-version: 2.0
+            library:
+              generated-other-modules: Paths_foo
+            |] `shouldRenderTo` (library [i|
+            other-modules:
+                Paths_foo
+            autogen-modules:
+                Paths_foo
+            |]) { packageCabalVersion = "2.0" }
+
+        context "when Paths_ module is listed explicitly under generated-exposed-modules" $ do
+          it "adds Paths_ to autogen-modules only once" $ do
+            [i|
+            verbatim:
+              cabal-version: 2.0
+            library:
+              generated-exposed-modules: Paths_foo
+            |] `shouldRenderTo` (library [i|
+            exposed-modules:
+                Paths_foo
+            autogen-modules:
+                Paths_foo
+            |]) { packageCabalVersion = "2.0" }
+
+      context "when Paths_ is mentioned in a conditional that is always false" $ do
+        it "does not add Paths_" $ do
+          [i|
+          library:
+            when:
+            - condition: false
+              other-modules: Paths_foo
+          |] `shouldRenderTo` library [i|
+          |]
+
+      context "with RebindableSyntax and OverloadedStrings or OverloadedStrings" $ do
+        it "infers cabal-version 2.2" $ do
+          [i|
+          default-extensions: [RebindableSyntax, OverloadedStrings]
+          library: {}
+          |] `shouldRenderTo` (library [i|
+          default-extensions:
+              RebindableSyntax
+              OverloadedStrings
+          other-modules:
+              Paths_foo
+          autogen-modules:
+              Paths_foo
+          |]) {packageCabalVersion = "2.2"}
+
+        context "when Paths_ is mentioned in a conditional that is always false" $ do
+          it "does not infer cabal-version 2.2" $ do
+            [i|
+            default-extensions: [RebindableSyntax, OverloadedStrings]
+            library:
+              when:
+              - condition: false
+                other-modules: Paths_foo
+            |] `shouldRenderTo` (library [i|
+            default-extensions:
+                RebindableSyntax
+                OverloadedStrings
+            |])
+
     describe "spec-version" $ do
       it "accepts spec-version" $ do
         [i|
@@ -215,7 +326,9 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           ref: "2017"
         library: {}
         |] `shouldRenderTo` library_ [i|
-        default-extensions: RecordWildCards DeriveFunctor
+        default-extensions:
+            RecordWildCards
+            DeriveFunctor
         |]
 
       it "accepts library defaults" $ do
@@ -245,7 +358,9 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           - foo/bar@v2
         library: {}
         |] `shouldRenderTo` library_ [i|
-        default-extensions: RecordWildCards DeriveFunctor
+        default-extensions:
+            RecordWildCards
+            DeriveFunctor
         |]
 
       it "accepts defaults recursively" $ do
@@ -255,7 +370,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
         defaults: foo/bar@v1
         library: {}
         |] `shouldRenderTo` library_ [i|
-        default-extensions: DeriveFunctor
+        default-extensions:
+            DeriveFunctor
         |]
 
       it "fails on cyclic defaults" $ do
@@ -326,7 +442,9 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
         |] `shouldRenderTo` library [i|
         other-modules:
             Paths_foo
-        default-extensions: RecordWildCards DeriveFunctor
+        default-extensions:
+            RecordWildCards
+            DeriveFunctor
         |]
 
     describe "version" $ do
@@ -380,6 +498,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           library
             other-modules:
                 Paths_foo
+            autogen-modules:
+                Paths_foo
             cxx-options: -Wall
             default-language: Haskell2010
           |]) {packageCabalVersion = "2.2"}
@@ -394,6 +514,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
           library
             other-modules:
                 Paths_foo
+            autogen-modules:
+                Paths_foo
             cxx-options: -Wall
             default-language: Haskell2010
           |]) {packageCabalVersion = "2.2"}
@@ -596,6 +718,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             system-build-tools:
               g++ >= 5.4.0
           |] `shouldRenderTo` (executable_ "foo" [i|
+          autogen-modules:
+              Paths_foo
           build-tools:
               g++ >=5.4.0
           |]) {packageCabalVersion = "2.0"}
@@ -615,6 +739,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
         executable:
           dependencies: foo:bar
         |] `shouldRenderTo` (executable_ "foo" [i|
+        autogen-modules:
+            Paths_foo
         build-depends:
             foo:bar
         |]) {packageCabalVersion = "3.0"}
@@ -725,6 +851,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
         executable:
           cxx-options: -Wall
         |] `shouldRenderTo` (executable_ "foo" [i|
+        autogen-modules:
+            Paths_foo
         cxx-options: -Wall
         |]) {packageCabalVersion = "2.2"}
 
@@ -740,6 +868,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                   condition: True
                   cxx-options: -Wall
           |] `shouldRenderTo` (executable_ "foo" [i|
+          autogen-modules:
+              Paths_foo
           if true
             if true
               if true
@@ -754,6 +884,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             - foo.cc
             - cxxbits/*.cc
         |] `shouldRenderTo` (executable_ "foo" [i|
+        autogen-modules:
+            Paths_foo
         cxx-sources:
             foo.cc
             cxxbits/bar.cc
@@ -904,6 +1036,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
         library:
           signatures: Foo
         |] `shouldRenderTo` (library_ [i|
+          autogen-modules:
+              Paths_foo
           signatures:
               Foo
         |]) {packageCabalVersion = "2.0"}
@@ -922,7 +1056,104 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
               Foo
           |]
 
-      context "when inferring modules" $ do
+      context "with mixins" $ do
+        it "infers cabal-version 2.0" $ do
+          [i|
+          library:
+            dependencies:
+              foo:
+                mixin:
+                  - (Blah as Etc)
+          |] `shouldRenderTo` (library [i|
+          other-modules:
+              Paths_foo
+          autogen-modules:
+              Paths_foo
+          build-depends:
+              foo
+          mixins:
+              foo (Blah as Etc)
+          |]) {packageCabalVersion = "2.0"}
+
+    describe "internal-libraries" $ do
+      it "accepts internal-libraries" $ do
+        touch "src/Foo.hs"
+        [i|
+        internal-libraries:
+          bar:
+            source-dirs: src
+        |] `shouldRenderTo` internalLibrary "bar" [i|
+        exposed-modules:
+            Foo
+        other-modules:
+            Paths_foo
+        autogen-modules:
+            Paths_foo
+        hs-source-dirs:
+            src
+        |]
+
+      it "warns on unknown fields" $ do
+        [i|
+        name: foo
+        internal-libraries:
+          bar:
+            baz: 42
+        |] `shouldWarn` pure "package.yaml: Ignoring unrecognized field $.internal-libraries.bar.baz"
+
+      it "warns on missing source-dirs" $ do
+        [i|
+        name: foo
+        internal-libraries:
+          bar:
+            source-dirs: src
+        |] `shouldWarn` pure "Specified source-dir \"src\" does not exist"
+
+      it "accepts visibility" $ do
+        [i|
+        internal-libraries:
+          bar:
+            visibility: public
+        |] `shouldRenderTo` (internalLibrary "bar" [i|
+        visibility: public
+        other-modules:
+            Paths_foo
+        autogen-modules:
+            Paths_foo
+        |]) {packageCabalVersion = "3.0"}
+
+    context "when inferring modules" $ do
+      context "with a library" $ do
+        it "ignores duplicate source directories" $ do
+          touch "src/Foo.hs"
+          [i|
+          source-dirs: src
+          library:
+            source-dirs: src
+          |] `shouldRenderTo` library [i|
+          hs-source-dirs:
+              src
+          exposed-modules:
+              Foo
+          other-modules:
+              Paths_foo
+          |]
+
+        it "ignores duplicate modules" $ do
+          touch "src/Foo.hs"
+          touch "src/Foo.x"
+          [i|
+          library:
+            source-dirs: src
+          |] `shouldRenderTo` library [i|
+          hs-source-dirs:
+              src
+          exposed-modules:
+              Foo
+          other-modules:
+              Paths_foo
+          |]
+
         context "with exposed-modules" $ do
           it "infers other-modules" $ do
             touch "src/Foo.hs"
@@ -1073,6 +1304,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Paths_foo
                 Bar
             autogen-modules:
+                Paths_foo
                 Foo
                 Bar
             |]) {packageCabalVersion = "2.0"}
@@ -1094,6 +1326,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Paths_foo
                 Other
             autogen-modules:
+                Paths_foo
                 Exposed
                 Other
             |]) {packageCabalVersion = "2.0"}
@@ -1111,6 +1344,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             |] `shouldRenderTo` (library [i|
             other-modules:
                 Paths_foo
+            autogen-modules:
+                Paths_foo
             hs-source-dirs:
                 src
             if os(windows)
@@ -1123,86 +1358,7 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                   Exposed
             |]) {packageCabalVersion = "2.0"}
 
-      context "mixins" $ do
-        it "sets cabal-version to 2.0 if mixins are used" $ do
-          [i|
-          library:
-            dependencies:
-              foo:
-                mixin:
-                  - (Blah as Etc)
-          |] `shouldRenderTo` (library [i|
-          other-modules:
-              Paths_foo
-          build-depends:
-              foo
-          mixins:
-              foo (Blah as Etc)
-          |]) {packageCabalVersion = "2.0"}
-
-    describe "internal-libraries" $ do
-      it "accepts internal-libraries" $ do
-        touch "src/Foo.hs"
-        [i|
-        internal-libraries:
-          bar:
-            source-dirs: src
-        |] `shouldRenderTo` internalLibrary "bar" [i|
-        exposed-modules:
-            Foo
-        other-modules:
-            Paths_foo
-        hs-source-dirs:
-            src
-        |]
-
-      it "warns on unknown fields" $ do
-        [i|
-        name: foo
-        internal-libraries:
-          bar:
-            baz: 42
-        |] `shouldWarn` pure "package.yaml: Ignoring unrecognized field $.internal-libraries.bar.baz"
-
-      it "warns on missing source-dirs" $ do
-        [i|
-        name: foo
-        internal-libraries:
-          bar:
-            source-dirs: src
-        |] `shouldWarn` pure "Specified source-dir \"src\" does not exist"
-
-      it "accepts visibility" $ do
-        [i|
-        internal-libraries:
-          bar:
-            visibility: public
-        |] `shouldRenderTo` (internalLibrary "bar" [i|
-        visibility: public
-        other-modules:
-            Paths_foo
-        |]) {packageCabalVersion = "3.0"}
-
-    describe "executables" $ do
-      it "accepts arbitrary entry points as main" $ do
-        touch "src/Foo.hs"
-        touch "src/Bar.hs"
-        [i|
-        executables:
-          foo:
-            source-dirs: src
-            main: Foo
-        |] `shouldRenderTo` executable "foo" [i|
-        main-is: Foo.hs
-        ghc-options: -main-is Foo
-        hs-source-dirs:
-            src
-        other-modules:
-            Bar
-            Paths_foo
-        |]
-
-      context "when inferring modules" $ do
+      context "with an executable" $ do
         it "infers other-modules" $ do
           touch "src/Main.hs"
           touch "src/Foo.hs"
@@ -1253,10 +1409,11 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 Paths_foo
                 Foo
             autogen-modules:
+                Paths_foo
                 Foo
           |]) {packageCabalVersion = "2.0"}
 
-        context "with conditional" $ do
+        context "with a conditional" $ do
           it "doesn't infer any modules mentioned in that conditional" $ do
             touch "src/Foo.hs"
             touch "src/Bar.hs"
@@ -1301,7 +1458,26 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                   windows
             |]
 
-      context "with conditional" $ do
+    describe "executables" $ do
+      it "accepts arbitrary entry points as main" $ do
+        touch "src/Foo.hs"
+        touch "src/Bar.hs"
+        [i|
+        executables:
+          foo:
+            source-dirs: src
+            main: Foo
+        |] `shouldRenderTo` executable "foo" [i|
+        main-is: Foo.hs
+        ghc-options: -main-is Foo
+        hs-source-dirs:
+            src
+        other-modules:
+            Bar
+            Paths_foo
+        |]
+
+      context "with a conditional" $ do
         it "does not apply global options" $ do
           -- related bug: https://github.com/sol/hpack/issues/214
           [i|
@@ -1380,6 +1556,40 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
                 unix
           |]
 
+        context "with empty then-branch" $ do
+          it "provides a hint" $ do
+            [i|
+            when:
+              condition: os(windows)
+              then: {}
+              else:
+                dependencies: unix
+            executable: {}
+            |] `shouldFailWith` unlines [
+                "package.yaml: Error while parsing $.when - an empty \"then\" section is not allowed, try the following instead:"
+              , ""
+              , "when:"
+              , "  condition: '!(os(windows))'"
+              , "  dependencies: unix"
+              ]
+
+        context "with empty else-branch" $ do
+          it "provides a hint" $ do
+            [i|
+            when:
+              condition: os(windows)
+              then:
+                dependencies: Win32
+              else: {}
+            executable: {}
+            |] `shouldFailWith` unlines [
+                "package.yaml: Error while parsing $.when - an empty \"else\" section is not allowed, try the following instead:"
+              , ""
+              , "when:"
+              , "  condition: os(windows)"
+              , "  dependencies: Win32"
+              ]
+
         it "rejects invalid conditionals" $ do
           [i|
           when:
@@ -1407,7 +1617,8 @@ spec = around_ (inTempDirectoryNamed "foo") $ do
             else:
               when:
                 condition: os(windows)
-                then: {}
+                then:
+                  dependencies: foo
                 else:
                   baz: null
           |] `shouldWarn` [
