@@ -19,7 +19,6 @@ import           Helper
 import           Data.Aeson.Config.FromValueSpec hiding (spec)
 
 import           Data.String.Interpolate.IsString
-import           Control.Arrow
 import qualified GHC.Exts as Exts
 import           System.Directory (createDirectory)
 import           Data.Either
@@ -148,25 +147,6 @@ spec = do
             }
       renameDependencies "bar" "baz" (sectionWithConditional ["foo", "bar"]) `shouldBe` sectionWithConditional ["foo", "baz"]
 
-  describe "getModules" $ around withTempDirectory $ do
-    it "returns Haskell modules in specified source directory" $ \dir -> do
-      touch (dir </> "src/Foo.hs")
-      touch (dir </> "src/Bar/Baz.hs")
-      touch (dir </> "src/Setup.hs")
-      getModules dir "src" >>= (`shouldMatchList` ["Foo", "Bar.Baz", "Setup"])
-
-    context "when source directory is '.'" $ do
-      it "ignores Setup" $ \dir -> do
-        touch (dir </> "Foo.hs")
-        touch (dir </> "Setup.hs")
-        getModules dir  "." `shouldReturn` ["Foo"]
-
-    context "when source directory is './.'" $ do
-      it "ignores Setup" $ \dir -> do
-        touch (dir </> "Foo.hs")
-        touch (dir </> "Setup.hs")
-        getModules dir  "./." `shouldReturn` ["Foo"]
-
   describe "toBuildTool" $ do
     let toBuildTool_ name = runWriter $ toBuildTool "my-package" ["foo"] (name, anyVersion)
     context "with an UnqualifiedBuildTool" $ do
@@ -662,17 +642,17 @@ spec = do
       it "accepts Strings" $ do
         [yaml|
         os(windows)
-        |] `shouldDecodeTo_` Cond "os(windows)"
+        |] `shouldDecodeTo_` CondExpression "os(windows)"
 
       it "accepts True" $ do
         [yaml|
         yes
-        |] `shouldDecodeTo_` Cond "true"
+        |] `shouldDecodeTo_` CondBool True
 
       it "accepts False" $ do
         [yaml|
         no
-        |] `shouldDecodeTo_` Cond "false"
+        |] `shouldDecodeTo_` CondBool False
 
       it "rejects other values" $ do
         [yaml|
