@@ -3,7 +3,6 @@
 module Hpack.RenderSpec (spec) where
 
 import           Helper
-import           Data.List
 
 import           Hpack.Syntax.DependencyVersion
 import           Hpack.ConfigSpec hiding (spec)
@@ -342,7 +341,7 @@ spec = do
       (render defaultRenderSettings 0 $ renderDirectories "name" ["."])
         `shouldBe` [
             "name:"
-          , "    ./."
+          , "    ./"
           ]
 
   describe "renderDependencies" $ do
